import { SqlDatabase } from './SqlDatabase';
import { ForeignKey } from './Table';
/**
 *
 *
 * @export
 * @class BaseDAO
 * @template T - The class mapped to the base table
 */
export declare class BaseDAO<T extends Object> {
    private type;
    private table;
    private sqldb;
    /**
     * Creates an instance of BaseDAO.
     *
     * @param {{new (): T}} type - The class mapped to the base table
     * @param {SqlDatabase} sqldb - The database connection
     */
    constructor(type: {
        new (): T;
    }, sqldb: SqlDatabase);
    /**
     * insert
     *
     * @param {T} t
     * @returns {Promise<T>}
     */
    insert(t: T): Promise<T>;
    /**
     * update
     *
     * @param {T} t
     * @returns {Promise<T>}
     */
    update(t: T): Promise<T>;
    /**
     * delete
     *
     * @param {T} t
     * @returns {Promise<void>}
     */
    delete(t: T): Promise<void>;
    /**
     * select using primary key
     *
     * @param {T} t
     * @returns {Promise<T>}
     */
    select(t: T): Promise<T>;
    /**
     * perform:
     * select T.<col1>,.. FROM <table> T
     *
     * @param {string} [sql] - An optional sql-text which will be added to the select-statement
     * @param {Object} [params] - An optional object with additional host parameter
     * @returns {Promise<Array<T>>}
     */
    selectAll(sql?: string, params?: Object): Promise<Array<T>>;
    /**
     * perform:
     * select T.<col1>,.. FROM <table> T
     *
     * @param {(err: Error, t: T) => void} callback - The callback called for each row
     * @param {string} [sql] - An optional sql-text which will be added to the select-statement
     * @param {Object} [params] - An optional object with additional host parameter
     * @returns {Promise<number>}
     */
    selectEach(callback: (err: Error, t: T) => void, sql?: string, params?: Object): Promise<number>;
    /**
     *
     * @template F - The class mapped to the foreign table
     * @param {string} constraintName - The foreign key constraint
     * @param {{new (): F}} foreignType - The class mapped to the foreign table
     * @param {F} foreign - An instance of the class mapped to the foreign table
     * @param {string} [sql] - An optional sql-text which will be added to the select-statement
     * @param {Object} [params] - An optional object with additional host parameter
     * @returns {Promise<Array<T>>}
     */
    selectAllOf<F extends Object>(constraintName: string, foreignType: {
        new (): F;
    }, foreign: F, sql?: string, params?: Object): Promise<Array<T>>;
    protected bindAllInputParams(t: T): Object;
    protected bindPrimaryKeyInputParams(t: T): Object;
    protected bindNonPrimaryKeyInputParams(t: T): Object;
    protected bindForeignParams<F extends Object>(foreignDAO: BaseDAO<F>, fk: ForeignKey, foreignObject: F, more?: Object): Object;
    protected readResultRow(t: T, row: any): T;
}
