"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const decorators_1 = require('./decorators');
/**
 *
 *
 * @export
 * @class BaseDAO
 * @template T - The class mapped to the base table
 */
class BaseDAO {
    /**
     * Creates an instance of BaseDAO.
     *
     * @param {{new (): T}} type - The class mapped to the base table
     * @param {SqlDatabase} sqldb - The database connection
     */
    constructor(type, sqldb) {
        this.type = type;
        this.table = Reflect.getMetadata(decorators_1.METADATA_TABLE_KEY, type.prototype);
        if (!this.table) {
            throw new Error(`no table-definition defined on prototype of ${this.type.name}'`);
        }
        this.sqldb = sqldb;
    }
    /**
     * insert
     *
     * @param {T} t
     * @returns {Promise<T>}
     */
    insert(t) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                if (!this.table.autoIncrementPropertyKey) {
                    let res = yield this.sqldb.run(this.table.getInsertIntoStatement(), this.bindAllInputParams(t));
                }
                else {
                    let res = yield this.sqldb.run(this.table.getInsertIntoStatement(), this.bindNonPrimaryKeyInputParams(t));
                    t[this.table.autoIncrementPropertyKey] = res.lastID;
                }
            }
            catch (e) {
                reject(new Error(`insert into '${this.table.name}' failed: ${e.message}`));
                return;
            }
            resolve(t);
        }));
    }
    ;
    /**
     * update
     *
     * @param {T} t
     * @returns {Promise<T>}
     */
    update(t) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                let res = yield this.sqldb.run(this.table.getUpdateSetStatement(), this.bindAllInputParams(t));
                if (!res.changes) {
                    reject(new Error(`update '${this.table.name}' failed: nothing changed`));
                    return;
                }
            }
            catch (e) {
                reject(new Error(`update '${this.table.name}' failed: ${e.message}`));
                return;
            }
            resolve(t);
        }));
    }
    ;
    /**
     * delete
     *
     * @param {T} t
     * @returns {Promise<void>}
     */
    delete(t) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                let res = yield this.sqldb.run(this.table.getDeleteFromStatement(), this.bindPrimaryKeyInputParams(t));
                if (!res.changes) {
                    reject(new Error(`delete from '${this.table.name}' failed: nothing changed`));
                    return;
                }
            }
            catch (e) {
                reject(new Error(`delete from '${this.table.name}' failed: ${e.message}`));
                return;
            }
            resolve();
        }));
    }
    ;
    /**
     * select using primary key
     *
     * @param {T} t
     * @returns {Promise<T>}
     */
    select(t) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                let row = yield this.sqldb.get(this.table.getSelectOneStatement(), this.bindPrimaryKeyInputParams(t));
                t = this.readResultRow(t, row);
            }
            catch (e) {
                reject(new Error(`select '${this.table.name}' failed: ${e.message}`));
                return;
            }
            resolve(t);
        }));
    }
    ;
    /**
     * perform:
     * select T.<col1>,.. FROM <table> T
     *
     * @param {string} [sql] - An optional sql-text which will be added to the select-statement
     * @param {Object} [params] - An optional object with additional host parameter
     * @returns {Promise<Array<T>>}
     */
    selectAll(sql, params) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                let stmt = this.table.getSelectAllStatement();
                if (!!sql) {
                    stmt += sql;
                }
                let rows = yield this.sqldb.all(stmt, params);
                let results = new Array();
                rows.forEach((row) => {
                    results.push(this.readResultRow(new this.type(), row));
                });
                resolve(results);
            }
            catch (e) {
                reject(new Error(`select '${this.table.name}' failed: ${e.message}`));
                return;
            }
        }));
    }
    ;
    /**
     * perform:
     * select T.<col1>,.. FROM <table> T
     *
     * @param {(err: Error, t: T) => void} callback - The callback called for each row
     * @param {string} [sql] - An optional sql-text which will be added to the select-statement
     * @param {Object} [params] - An optional object with additional host parameter
     * @returns {Promise<number>}
     */
    selectEach(callback, sql, params) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                let stmt = this.table.getSelectAllStatement();
                if (!!sql) {
                    stmt += sql;
                }
                let res = yield this.sqldb.each(stmt, params, (err, row) => {
                    // TODO: err?
                    callback(err, this.readResultRow(new this.type(), row));
                });
                resolve(res);
            }
            catch (e) {
                reject(new Error(`select '${this.table.name}' failed: ${e.message}`));
                return;
            }
        }));
    }
    ;
    /**
     *
     * @template F - The class mapped to the foreign table
     * @param {string} constraintName - The foreign key constraint
     * @param {{new (): F}} foreignType - The class mapped to the foreign table
     * @param {F} foreign - An instance of the class mapped to the foreign table
     * @param {string} [sql] - An optional sql-text which will be added to the select-statement
     * @param {Object} [params] - An optional object with additional host parameter
     * @returns {Promise<Array<T>>}
     */
    selectAllOf(constraintName, foreignType, foreign, sql, params) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                if (!this.table.statementsText.foreignKeys.has(constraintName)) {
                    throw new Error(`constraint '${constraintName}' is not defined`);
                }
                let fk = this.table.statementsText.foreignKeys.get(constraintName);
                let stmt = this.table.getSelectAllStatement();
                stmt += '\nWHERE\n  ';
                stmt += fk.selectCondition;
                let foreignDAO = new BaseDAO(foreignType, this.sqldb);
                let foreignParams = this.bindForeignParams(foreignDAO, fk, foreign, params);
                if (!!sql) {
                    stmt += sql;
                }
                let rows = yield this.sqldb.all(stmt, foreignParams);
                let results = new Array();
                rows.forEach((row) => {
                    results.push(this.readResultRow(new this.type(), row));
                });
                resolve(results);
            }
            catch (e) {
                reject(new Error(`select '${this.table.name}' failed: ${e.message}`));
                return;
            }
        }));
    }
    ;
    bindAllInputParams(t) {
        let res = {};
        this.table.fields.forEach((field) => res[field.hostParameterName] =
            t[field.propertyKey]);
        return res;
    }
    bindPrimaryKeyInputParams(t) {
        let res = {};
        this.table.fields.forEach((field) => {
            if (field.isIdentity) {
                res[field.hostParameterName] = t[field.propertyKey];
            }
        });
        return res;
    }
    bindNonPrimaryKeyInputParams(t) {
        let res = {};
        this.table.fields.forEach((field) => {
            if (!field.isIdentity) {
                res[field.hostParameterName] = t[field.propertyKey];
            }
        });
        return res;
    }
    bindForeignParams(foreignDAO, fk, foreignObject, more = {}) {
        let res = Object.assign({}, more);
        fk.fields.forEach((field) => {
            let fieldref = field.foreignKeys.get(fk.name);
            let foreignfield = foreignDAO.table.getTableField(fieldref.colName);
            res[field.hostParameterName] =
                foreignObject[foreignfield.propertyKey];
        });
        return res;
    }
    readResultRow(t, row) {
        // TODO: ensure type safety
        this.table.fields.forEach((field) => t[field.propertyKey] = row[field.name]);
        return t;
    }
}
exports.BaseDAO = BaseDAO;
//# sourceMappingURL=BaseDAO.js.map