"use strict";
/**
 * A singleton holding the database schema definitions
 *
 * @export
 * @class Schema
 */
class Schema {
    /**
     * Creates an instance of Schema.
     *
     */
    constructor() {
        if (!Schema.schema) {
            // initialize the 'singleton'
            Schema.schema = this;
            this.mapNameToTable = new Map();
        }
        return Schema.schema;
    }
    /**
     * get a table definition
     *
     * @param {string} name - The name of the table
     * @returns {Table}
     */
    getTable(name) {
        if (!this.mapNameToTable.has(name)) {
            throw new Error(`table '${name}' not registered yet`);
        }
        return this.mapNameToTable.get(name);
    }
    /**
     * add a table definition
     *
     * @param {string} name - The name of the table
     * @param {Table} table
     * @returns {Table}
     */
    addTable(table) {
        if (this.mapNameToTable.has(table.name)) {
            throw new Error(`table '${table.name}' already registered`);
        }
        this.mapNameToTable.set(table.name, table);
        table.fields.forEach((field) => table.addTableField(field));
        return table;
    }
    /**
     * create a table in the database
     *
     * @param {SqlDatabase} sqldb
     * @param {string} name - The name of the table
     * @returns {Promise<void>}
     */
    createTable(sqldb, name) {
        return sqldb.exec(this.getTable(name).getCreateTableStatement());
    }
    /**
     * drop a table from the database
     *
     * @param {SqlDatabase} sqldb
     * @param {string} name - The name of the table
     * @returns {Promise<void>}
     */
    dropTable(sqldb, name) {
        return sqldb.exec(this.getTable(name).getDropTableStatement());
    }
    /**
     * add a column/field to a database table
     *
     * @param {SqlDatabase} sqldb
     * @param {string} tableName
     * @param {string} colName
     * @returns {Promise<void>}
     */
    alterTableAddColumn(sqldb, tableName, colName) {
        return sqldb.exec(this.getTable(tableName).getAlterTableAddColumnStatement(colName));
    }
}
exports.Schema = Schema;
/**
 * get the Schema singleton
 *
 * @export
 * @returns {Schema}
 */
function schema() {
    if (!Schema.schema) {
        let schema = new Schema();
    }
    return Schema.schema;
}
exports.schema = schema;
//# sourceMappingURL=Schema.js.map