import { SqlStatement, SqlRunResult } from './SqlStatement';
export declare const SQL_OPEN_READONLY: number;
export declare const SQL_OPEN_READWRITE: number;
export declare const SQL_OPEN_CREATE: number;
export declare const SQL_MEMORY_DB_SHARED: string;
export declare const SQL_MEMORY_DB_PRIVATE: string;
/**
 * A thin wrapper for the 'Database' class from 'node-sqlite3' using Promises
 * instead of callbacks
 * see
 * https://github.com/mapbox/node-sqlite3/wiki/API
 *
 * see why we may want to have a connection pool running on nodejs serving multiple requests
 * https://github.com/mapbox/node-sqlite3/issues/304
 *
 * @export
 * @class SqlDatabase
 */
export declare class SqlDatabase {
    private db;
    /**
     * Creates an instance of SqlDatabase.
     *
     */
    constructor();
    /**
     * Open a database connection
     *
     * @param {string} databaseFile - The path to the database file or URI
     * filename (see SQL_MEMORY_DB_SHARED/SQL_MEMORY_DB_PRIVATE for an in-memory
     * database)
     * @param {number} [mode] - A bit flag combination of: SQL_OPEN_CREATE |
     * SQL_OPEN_READONLY | SQL_OPEN_READWRITE
     * @returns {Promise<void>}
     */
    open(databaseFile: string, mode?: number): Promise<void>;
    /**
     * Close the database connection
     *
     * @returns {Promise<void>}
     */
    close(): Promise<void>;
    /**
     * Runs a SQL statement with the specified parameters
     *
     * @param {string} sql - The SQL statment
     * @param {*} [params] - The parameters referenced in the statement; you can
     * provide multiple parameters as array
     * @returns {Promise<SqlRunResult>}
     */
    run(sql: string, params?: any): Promise<SqlRunResult>;
    /**
     * Runs a SQL query with the specified parameters, fetching only the first row
     *
     * @param {string} sql - The DQL statement
     * @param {*} [params] - The parameters referenced in the statement; you can
     * provide multiple parameters as array
     * @returns {Promise<any>}
     */
    get(sql: string, params?: any): Promise<any>;
    /**
     * Runs a SQL query with the specified parameters, fetching all rows
     *
     * @param {string} sql - The DQL statement
     * @param {*} [params] - The parameters referenced in the statement; you can
     * provide multiple parameters as array
     * @returns {Promise<any[]>}
     */
    all(sql: string, params?: any): Promise<any[]>;
    /**
     * Runs a SQL query with the specified parameters, fetching all rows
     * using a callback for each row
     *
     * @param {string} sql - The DQL statement
     * @param {*} [params] - The parameters referenced in the statement; you can
     * provide multiple parameters as array
     * @param {(err: Error, row: any) => void} [callback]
     * @returns {Promise<number>}
     */
    each(sql: string, params?: any, callback?: (err: Error, row: any) => void): Promise<number>;
    /**
     * Execute a SQL statement
     *
     * @param {string} sql - The SQL statement
     * @returns {Promise<void>}
     */
    exec(sql: string): Promise<void>;
    /**
     * Prepare a SQL statement
     *
     * @param {string} sql - The SQL statement
     * @param {*} [params] - The parameters referenced in the statement; you can
     * provide multiple parameters as array
     * @returns {Promise<SqlStatement>}
     */
    prepare(sql: string, params?: any): Promise<SqlStatement>;
    /**
     * serialized sqlite3 calls
     * if callback is provided, run callback in serialized mode
     * otherwise, switch connection to serialized mode
     *
     * @param {() => void} [callback]
     * @returns {void}
     */
    serialize(callback?: () => void): void;
    /**
     * parallelized sqlite3 calls
     * if callback is provided, run callback in parallel mode
     * otherwise, switch connection to parallel mode
     *
     * @param {() => void} [callback]
     * @returns {void}
     */
    parallelize(callback?: () => void): void;
    /**
     * Run callback inside a database transaction
     *
     * @param {() => void} [callback]
     * @returns {void}
     */
    transactionalize<T>(callback: () => Promise<T>): Promise<T>;
    /**
     *
     *
     * @param {'trace'} event
     * @param {(sql: string) => void} listener
     * @returns {this}
     */
    on(event: 'trace', listener: (sql: string) => void): this;
    /**
     *
     *
     * @param {'profile'} event
     * @param {(sql: string) => void} listener
     * @returns {this}
     */
    on(event: 'profile', listener: (sql: string) => void): this;
    /**
     *
     *
     * @param {'error'} event
     * @param {(sql: string) => void} listener
     * @returns {this}
     */
    on(event: 'error', listener: (sql: string) => void): this;
    /**
     *
     *
     * @param {'open'} event
     * @param {(sql: string) => void} listener
     * @returns {this}
     */
    on(event: 'open', listener: (sql: string) => void): this;
    /**
     *
     *
     * @param {'close'} event
     * @param {(sql: string) => void} listener
     * @returns {this}
     */
    on(event: 'close', listener: (sql: string) => void): this;
    /**
     * Get the 'user_version' from the database
     *
     * @returns {Promise<number>}
     */
    getUserVersion(): Promise<number>;
    /**
     * Set the 'user_version' in the database
     *
     * @param {number} newver
     * @returns {Promise<void>}
     */
    setUserVersion(newver: number): Promise<void>;
    static verbose(): void;
}
