"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const sqlite3_1 = require('sqlite3');
const SqlStatement_1 = require('./SqlStatement');
exports.SQL_OPEN_READONLY = sqlite3_1.OPEN_READONLY;
exports.SQL_OPEN_READWRITE = sqlite3_1.OPEN_READWRITE;
exports.SQL_OPEN_CREATE = sqlite3_1.OPEN_CREATE;
exports.SQL_MEMORY_DB_SHARED = 'file::memory:?cache=shared';
exports.SQL_MEMORY_DB_PRIVATE = ':memory:';
// NOTE:
// our tests defined in 'SqlDatabase.spec.ts' are working fine using
// private-cache mode:
//   export const SQL_MEMORY_DB_PRIVATE = ':memory:';
//   export const SQL_MEMORY_DB_PRIVATE = 'file::memory:?cache=off';
// but failing, if private-cache mode is explicitly defined:
//   export const SQL_MEMORY_DB_PRIVATE = 'file::memory:?cache=private';
// could it be, that the last URI opens the memory in shared-cache mode?
// but even running jasmine using `jasmine /path/to/Database.spec.js` works as
// expected,
// only if jasmine is called without any spec.js, these tests are failing in the
// 'beforeAll' function, with:
//   Error: SQLITE_ERROR: table TEST already exists
//   Error: SQLITE_CONSTRAINT: UNIQUE constraint failed: TEST.id
const SQL_OPEN_DEFAULT = exports.SQL_OPEN_READWRITE | exports.SQL_OPEN_CREATE;
/**
 * A thin wrapper for the 'Database' class from 'node-sqlite3' using Promises
 * instead of callbacks
 * see
 * https://github.com/mapbox/node-sqlite3/wiki/API
 *
 * see why we may want to have a connection pool running on nodejs serving multiple requests
 * https://github.com/mapbox/node-sqlite3/issues/304
 *
 * @export
 * @class SqlDatabase
 */
class SqlDatabase {
    /**
     * Creates an instance of SqlDatabase.
     *
     */
    constructor() {
    }
    /**
     * Open a database connection
     *
     * @param {string} databaseFile - The path to the database file or URI
     * filename (see SQL_MEMORY_DB_SHARED/SQL_MEMORY_DB_PRIVATE for an in-memory
     * database)
     * @param {number} [mode] - A bit flag combination of: SQL_OPEN_CREATE |
     * SQL_OPEN_READONLY | SQL_OPEN_READWRITE
     * @returns {Promise<void>}
     */
    open(databaseFile, mode) {
        return new Promise((resolve, reject) => {
            this.db = new sqlite3_1.Database(databaseFile, mode || SQL_OPEN_DEFAULT, (err) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve();
                }
            });
        });
    }
    /**
     * Close the database connection
     *
     * @returns {Promise<void>}
     */
    close() {
        return new Promise((resolve, reject) => {
            if (!this.db) {
                resolve();
            }
            else {
                this.db.close((err) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve();
                    }
                });
            }
        });
    }
    /**
     * Runs a SQL statement with the specified parameters
     *
     * @param {string} sql - The SQL statment
     * @param {*} [params] - The parameters referenced in the statement; you can
     * provide multiple parameters as array
     * @returns {Promise<SqlRunResult>}
     */
    run(sql, params) {
        return new Promise((resolve, reject) => {
            // trace('run stmt=' + sql);
            // trace('>input: ' + JSON.stringify(params));
            // tslint:disable-next-line: only-arrow-functions
            this.db.run(sql, params, function (err) {
                if (err) {
                    reject(err);
                }
                else {
                    // tslint:disable-next-line: no-invalid-this
                    let res = { lastID: this.lastID, changes: this.changes };
                    resolve(res);
                }
            });
        });
    }
    /**
     * Runs a SQL query with the specified parameters, fetching only the first row
     *
     * @param {string} sql - The DQL statement
     * @param {*} [params] - The parameters referenced in the statement; you can
     * provide multiple parameters as array
     * @returns {Promise<any>}
     */
    get(sql, params) {
        return new Promise((resolve, reject) => {
            // trace('get stmt=' + sql);
            // trace('>input: ' + JSON.stringify(params));
            this.db.get(sql, params, (err, row) => {
                if (err) {
                    // trace('>error: ' + err.message);
                    reject(err);
                }
                else {
                    // trace('>succeeded: ' + JSON.stringify(row));
                    resolve(row);
                }
            });
        });
    }
    /**
     * Runs a SQL query with the specified parameters, fetching all rows
     *
     * @param {string} sql - The DQL statement
     * @param {*} [params] - The parameters referenced in the statement; you can
     * provide multiple parameters as array
     * @returns {Promise<any[]>}
     */
    all(sql, params) {
        return new Promise((resolve, reject) => {
            // trace('all stmt=' + sql);
            // trace('>input: ' + JSON.stringify(params));
            this.db.all(sql, params, (err, rows) => {
                if (err) {
                    // trace('>error: ' + err.message);
                    reject(err);
                }
                else {
                    // trace('>succeeded: ' + JSON.stringify(rows));
                    resolve(rows);
                }
            });
        });
    }
    /**
     * Runs a SQL query with the specified parameters, fetching all rows
     * using a callback for each row
     *
     * @param {string} sql - The DQL statement
     * @param {*} [params] - The parameters referenced in the statement; you can
     * provide multiple parameters as array
     * @param {(err: Error, row: any) => void} [callback]
     * @returns {Promise<number>}
     */
    each(sql, params, callback) {
        return new Promise((resolve, reject) => {
            this.db.each(sql, params, callback, (err, count) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(count);
                }
            });
        });
    }
    /**
     * Execute a SQL statement
     *
     * @param {string} sql - The SQL statement
     * @returns {Promise<void>}
     */
    exec(sql) {
        return new Promise((resolve, reject) => {
            // trace('exec stmt=' + sql);
            this.db.exec(sql, (err) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve();
                }
            });
        });
    }
    /**
     * Prepare a SQL statement
     *
     * @param {string} sql - The SQL statement
     * @param {*} [params] - The parameters referenced in the statement; you can
     * provide multiple parameters as array
     * @returns {Promise<SqlStatement>}
     */
    prepare(sql, params) {
        return new Promise((resolve, reject) => {
            let dbstmt;
            dbstmt = this.db.prepare(sql, params, (err) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(new SqlStatement_1.SqlStatement(dbstmt));
                }
            });
        });
    }
    /**
     * serialized sqlite3 calls
     * if callback is provided, run callback in serialized mode
     * otherwise, switch connection to serialized mode
     *
     * @param {() => void} [callback]
     * @returns {void}
     */
    serialize(callback) {
        return this.db.serialize(callback);
    }
    /**
     * parallelized sqlite3 calls
     * if callback is provided, run callback in parallel mode
     * otherwise, switch connection to parallel mode
     *
     * @param {() => void} [callback]
     * @returns {void}
     */
    parallelize(callback) {
        return this.db.parallelize(callback);
    }
    /**
     * Run callback inside a database transaction
     *
     * @param {() => void} [callback]
     * @returns {void}
     */
    transactionalize(callback) {
        return this.run('BEGIN IMMEDIATE TRANSACTION')
            .then(callback)
            .then((res) => this.run('COMMIT TRANSACTION').then(() => Promise.resolve(res)))
            .catch((err) => this.run('ROLLBACK TRANSACTION')
            .then(() => Promise.reject(err)));
    }
    /**
     *
     *
     * @param {string} event
     * @param {(sql: string) => void} listener
     * @returns {this}
     */
    on(event, listener) {
        this.db.on(event, listener);
        return this;
    }
    /**
     * Get the 'user_version' from the database
     *
     * @returns {Promise<number>}
     */
    getUserVersion() {
        return __awaiter(this, void 0, void 0, function* () {
            let userVersion = 0;
            try {
                let res = yield this.get('PRAGMA user_version');
                userVersion = res.user_version;
            }
            catch (e) {
                return Promise.reject(e);
            }
            return Promise.resolve(userVersion);
        });
    }
    /**
     * Set the 'user_version' in the database
     *
     * @param {number} newver
     * @returns {Promise<void>}
     */
    setUserVersion(newver) {
        return this.exec('PRAGMA user_version = ' + newver);
    }
    static verbose() { sqlite3_1.verbose(); }
}
exports.SqlDatabase = SqlDatabase;
//# sourceMappingURL=SqlDatabase.js.map