import { Field } from './Field';
/**
 * Class with the foreign key definition
 *
 * @export
 * @class Table
 */
export declare class ForeignKey {
    name: string;
    refTableName: string;
    refColumns: Array<string>;
    fields: Array<Field>;
    selectCondition: string;
    constructor(name: string, refTableName: string);
}
/**
 * Class with the sql statements
 *
 * @export
 * @class Table
 */
export declare class SqlStatementText {
    createTable: string;
    dropTable: string;
    alterTable: string;
    insertInto: string;
    updateSet: string;
    deleteFrom: string;
    selectAll: string;
    selectOne: string;
    foreignKeys: Map<string, ForeignKey>;
    constructor();
}
/**
 * Class holding a table definition (name of the table and fields in the table)
 *
 * @export
 * @class Table
 */
export declare class Table {
    /**
     * The table name
     *
     * @type {string}
     */
    name: string;
    /**
     * The class name
     *
     * @type {string}
     */
    className: string;
    /**
     * Flag to indicate if this table should be created with the 'WITHOUT
     * ROWID'-clause
     *
     * @type {boolean}
     */
    withoutRowId: boolean;
    /**
     * Flag to indicate if AUTOINCREMENT should be enabled for a table having a
     * single-column INTEGER primary key
     * and withoutRowId is disabled
     *
     * @type {boolean}
     */
    autoIncrement: boolean;
    /**
     * The fields defined for this table
     *
     * @type {Array<Field>}
     */
    fields: Array<Field>;
    /**
     * This contains the generated SQL-statements/fragments
     *
     * @type {SqlStatementText}
     */
    private _statementsText?;
    readonly statementsText: SqlStatementText;
    /**
     * The property key of the primary key, only set if the
     * AUTOINCREMENT feature can be used
     *
     * @type {string|symbol}
     */
    private _autoIncrementPropertyKey;
    readonly autoIncrementPropertyKey: string | symbol | undefined;
    private mapPropToField;
    private mapNameToField;
    private mapNameToIdentityField;
    /**
     * Creates an instance of Table.
     *
     * @param {string} className
     */
    constructor(className: string);
    /**
     * Test if property has been mapped to a column of this table
     *
     * @param {(string|symbol)} key - The property key
     * @returns {boolean}
     */
    hasPropertyField(key: string | symbol): boolean;
    /**
     * Get the field definition for the mapped property key
     *
     * @param {(string|symbol)} key - The property key
     * @returns {Field}
     */
    getPropertyField(key: string | symbol): Field;
    /**
     * Add a property key mapped to field definition to this table
     *
     * @param {(string|symbol)} key - The property key
     * @param {Field} field
     */
    addPropertyField(field: Field): void;
    /**
     * Test if table has a column with the given column name
     *
     * @param {string} colName - The name of the column
     * @returns {boolean}
     */
    hasTableField(name: string): boolean;
    /**
     * Get the field definition for the given column name
     *
     * @param {string} colName - The name of the column
     * @returns {Field}
     */
    getTableField(name: string): Field;
    /**
     * Add a table field to this table
     *
     * @param {string} colName - The name of the column
     * @returns {Field}
     */
    addTableField(field: Field): Field;
    /**
     * Get 'CREATE TABLE'-statement using 'IF NOT EXISTS'-clause
     *
     * @returns {string}
     */
    getCreateTableStatement(): string;
    /**
     * Get 'DROP TABLE'-statement
     *
     * @returns {string}
     */
    getDropTableStatement(): string;
    /**
     * Get 'ALTER TABLE...ADD COLUMN'-statement for the given column
     *
     * @param {string} colName - The name of the column to add to the table
     * @returns {string}
     */
    getAlterTableAddColumnStatement(colName: string): string;
    /**
     * Get 'INSERT INTO'-statement
     *
     * @returns {string}
     */
    getInsertIntoStatement(): string;
    /**
     * Get 'UPDATE SET'-statement
     *
     * @returns {string}
     */
    getUpdateSetStatement(): string;
    /**
     * Get 'DELETE FROM'-statement
     *
     * @returns {string}
     */
    getDeleteFromStatement(): string;
    /**
     * Get 'SELECT' all-statement
     *
     * @returns {string}
     */
    getSelectAllStatement(): string;
    /**
     * Get 'SELECT' one-statement
     *
     * @returns {string}
     */
    getSelectOneStatement(): string;
    /**
     * Generate SQL Statements
     *
     */
    private generateStatementsText();
}
