import 'reflect-metadata';
export declare const METADATA_TABLE_KEY: string;
/**
 * Options for the '@table' class decorator
 *
 * @export
 * @interface TableOpts
 */
export interface TableOpts {
    /**
     * The name of the table
     * @type {string}
     */
    name?: string;
    /**
     * Flag to indicate if table should be created using the 'WITHOUT ROWID' clause
     * @type {boolean}
     */
    withoutRowId?: boolean;
    /**
     * Flag to indicate if AUTOINCREMENT should be added to single-column INTEGER primary keys
     * @type {boolean}
     */
    autoIncrement?: boolean;
}
/**
 * Options for the property decorators '@field' and '@id'
 *
 * @export
 * @interface FieldOpts
 */
export interface FieldOpts {
    /**
     * The name of the table field
     * @type {string}
     */
    name?: string;
    /**
     * The column definition
     * @type {string}
     */
    dbtype?: string;
}
/**
 * The class decorator for mapping a database table to a class
 *
 * @export
 * @param {TableOpts} [opts]
 * @returns {(target: Function) => void}
 */
export declare function table(opts?: TableOpts): (target: Function) => void;
/**
 * The field decorator for mapping a class property to a table field
 *
 * @export
 * @param {string} [name] - The name of the field; defaults to the property name
 * @param {string} [dbtype] - The type of the field; defaults to 'TEXT'
 * @returns {((
 *     target: Object, key: string|symbol) => void)}
 */
export declare function field(opts?: FieldOpts): (target: Object, key: string | symbol) => void;
/**
 * The id decorator for mapping a class property to a field of the primary key of the table
 *
 * @export
 * @param {string} [name] - The name of the field; defaults to the property name
 * @param {string} [dbtype] - The type of the field; defaults to 'TEXT'
 * @returns {((
 *     target: Object, key: string|symbol) => void)}
 */
export declare function id(opts?: FieldOpts): (target: Object, key: string | symbol) => void;
/**
 * The fk decorator for mapping a class property to be part of a foreign key constraint
 *
 * @export
 * @param {string} constraintName - The constraint name
 * @param {string} foreignTableName - The referenced table name
 * @param {string} foreignTableField - The referenced table field
 * @returns {((target: Object, key: string|symbol) =>
 *     void)}
 */
export declare function fk(constraintName: string, foreignTableName: string, foreignTableField: string): (target: Object, key: string | symbol) => void;
