"use strict";
require('reflect-metadata');
const Field_1 = require('./Field');
const Schema_1 = require('./Schema');
const Table_1 = require('./Table');
exports.METADATA_TABLE_KEY = 'schema:table';
/**
 * Get table metadata
 *
 * @param {Function} target - The constructor of the class
 * @returns {Table}
 */
function getTableMetadata(target) {
    if (!Reflect.hasOwnMetadata(exports.METADATA_TABLE_KEY, target.prototype)) {
        Reflect.defineMetadata(exports.METADATA_TABLE_KEY, new Table_1.Table(target.name), target.prototype);
    }
    let table = Reflect.getMetadata(exports.METADATA_TABLE_KEY, target.prototype);
    return table;
}
function getFieldMetadata(table, key) {
    let field;
    if (table.hasPropertyField(key)) {
        field = table.getPropertyField(key);
    }
    else {
        field = new Field_1.Field(key);
        table.addPropertyField(field);
    }
    return field;
}
/**
 * Helper function for decorating a class and map it to a database table
 *
 * @param {Function} target - The constructor of the class
 * @param {TableOpts} [opts]
 * @returns {Table}
 */
function decorateClass(target, opts) {
    let table = getTableMetadata(target);
    if (!!opts.name && !!table.name && name !== table.name) {
        throw new Error(`failed to map class '${target.name}' to table name '${opts.name}': This class is already mapped to the table '${table.name}'`);
    }
    table.name = opts.name || target.name;
    if (!!opts.withoutRowId) {
        table.withoutRowId = true;
    }
    if (!!opts.autoIncrement) {
        table.autoIncrement = true;
    }
    Schema_1.schema().addTable(table);
}
/**
 * Helper function for decorating a property and map it to a table field
 *
 * @param {Object} target - The decorated class
 * @param {(string|symbol)} key - The decorated property
 * @param {FieldOpts} [opts]
 * @param {boolean} [isIdentity=false] - Indicator if this field belongs to the primary key
 * @returns {Field}
 */
function decoratePropertyField(target, key, opts, isIdentity = false) {
    if (typeof target === 'function') {
        // not decorating static property
        throw new Error(`decorating static property '${key.toString()}' using field-decorator is not supported`);
    }
    let table = getTableMetadata(target.constructor);
    let field = getFieldMetadata(table, key);
    field.propertyType = Reflect.getMetadata('design:type', target, key);
    field.name = opts.name || key.toString();
    if (!!opts.dbtype) {
        field.dbtype = opts.dbtype;
    }
    if (!!isIdentity) {
        field.isIdentity = isIdentity;
    }
    return field;
}
/**
 * Helper function for decorating a property and map it to a foreign key field
 *
 * @param {Object} target - The decorated class
 * @param {(string|symbol)} key - The decorated property
 * @param {string} constraintName - The name for the foreign key constraint
 * @param {string} foreignTableName - The referenced table name
 * @param {string} foreignTableField - The referenced table field
 * @returns {Field}
 */
function decoratePropertyForeignKey(target, key, constraintName, foreignTableName, foreignTableField) {
    if (typeof target === 'function') {
        // not decorating static property
        throw new Error(`decorating static property '${key.toString()}' using field-decorator is not supported`);
    }
    let table = getTableMetadata(target.constructor);
    let field = getFieldMetadata(table, key);
    if (field.hasForeignKeyConstraint(constraintName)) {
        throw new Error(`decorating property '${target.constructor.name}.${key.toString()}': duplicate foreign key constraint 'constraintName'`);
    }
    field.setForeignKeyField(constraintName, new Field_1.FieldReference(foreignTableName, foreignTableField));
    return field;
}
/**
 * The class decorator for mapping a database table to a class
 *
 * @export
 * @param {TableOpts} [opts]
 * @returns {(target: Function) => void}
 */
function table(opts = {}) {
    return ((target) => decorateClass(target, opts));
}
exports.table = table;
/**
 * The field decorator for mapping a class property to a table field
 *
 * @export
 * @param {string} [name] - The name of the field; defaults to the property name
 * @param {string} [dbtype] - The type of the field; defaults to 'TEXT'
 * @returns {((
 *     target: Object, key: string|symbol) => void)}
 */
function field(opts = {}) {
    return ((target, key) => {
        decoratePropertyField(target, key, opts, false);
    });
}
exports.field = field;
/**
 * The id decorator for mapping a class property to a field of the primary key of the table
 *
 * @export
 * @param {string} [name] - The name of the field; defaults to the property name
 * @param {string} [dbtype] - The type of the field; defaults to 'TEXT'
 * @returns {((
 *     target: Object, key: string|symbol) => void)}
 */
function id(opts = {}) {
    return ((target, key) => {
        decoratePropertyField(target, key, opts, true);
    });
}
exports.id = id;
/**
 * The fk decorator for mapping a class property to be part of a foreign key constraint
 *
 * @export
 * @param {string} constraintName - The constraint name
 * @param {string} foreignTableName - The referenced table name
 * @param {string} foreignTableField - The referenced table field
 * @returns {((target: Object, key: string|symbol) =>
 *     void)}
 */
function fk(constraintName, foreignTableName, foreignTableField) {
    return ((target, key) => {
        let field = decoratePropertyForeignKey(target, key, constraintName, foreignTableName, foreignTableField);
    });
}
exports.fk = fk;
//# sourceMappingURL=decorators.js.map